package com.brtchip.uart2gpio.ui;

import javax.swing.JFrame;
import javax.swing.JScrollPane;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.concurrent.CountDownLatch;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JTextPane;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.ScrollPaneConstants;
import javax.swing.filechooser.FileNameExtensionFilter;

public class ScriptUI {

	private JFrame frame;
	private JTextPane textPane;
	private String action;
	
	private final CountDownLatch latchClose = new CountDownLatch(1);

	private final Action actBtnClicker = new ButtonClickerAction();

	/**
	 * Create the application.
	 */
	public ScriptUI(JFrame parent) {
		initialize(parent);
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize(JFrame parent) {
		frame = new JFrame();
		frame.setIconImage(parent.getIconImage());
		frame.setTitle("Script Console");
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		frame.setResizable(false);
		frame.setBounds(100, 100, 350, 300);
		frame.setLocationRelativeTo(null);

		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		frame.getContentPane().add(scrollPane, BorderLayout.CENTER);
		
		textPane = new JTextPane();
		textPane.setFont(textPane.getFont().deriveFont(textPane.getFont().getSize() + 5f));
		scrollPane.setViewportView(textPane);
		
		JPanel panelButtons = new JPanel();
		frame.getContentPane().add(panelButtons, BorderLayout.SOUTH);
		
		ActionListener al = new ButtonActionListener();
				
		JButton btnOk = new JButton("Send");
		btnOk.setActionCommand("OK");
		btnOk.addActionListener(al);
		addKeyShortcut(btnOk, KeyEvent.VK_ENTER, InputEvent.CTRL_MASK);
		panelButtons.add(btnOk);
		
		JButton btnCancel = new JButton("Cancel");
		btnCancel.setActionCommand("Cancel");
		btnCancel.addActionListener(al);
		addKeyShortcut(btnCancel, KeyEvent.VK_ESCAPE, 0);
		panelButtons.add(btnCancel);
		
		JButton btnLoad = new JButton("Load");
		btnLoad.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				JFileChooser fc = new JFileChooser();
				FileNameExtensionFilter ffiltText = new FileNameExtensionFilter("Text Files (*.txt)", "txt");
				fc.addChoosableFileFilter(ffiltText);
				fc.setFileFilter(ffiltText);
				fc.setFileSelectionMode(JFileChooser.FILES_ONLY);

				int ret = fc.showOpenDialog(null);
				
				if (ret == JFileChooser.APPROVE_OPTION)
				{
					File f = fc.getSelectedFile();
					try {
						System.out.println(e.getActionCommand()+": Opening "+f);
						FileReader fr = new FileReader(f);
						BufferedReader br = new BufferedReader(fr);
						textPane.read(br, null);
						br.close();
					} catch (FileNotFoundException ex) {
						System.out.println(e.getActionCommand()+": Cannot find file "+f);
					} catch (IOException ex) {
						ex.printStackTrace();
					}
					
				}
			}
		});
		panelButtons.add(btnLoad);
	}
	
	public String getText()
	{
		frame.setVisible(true);
		
		try {
			latchClose.await();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		if (action=="OK")
		{
			return textPane.getText();
		}
		return null;
	}
	
	private void addKeyShortcut(JButton but, int key, int mods)
	{
		KeyStroke ks = KeyStroke.getKeyStroke(key, mods, false);
		InputMap im = but.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap am = but.getActionMap();
		String actstr = but.getActionCommand()+" Action";
		
		im.put(ks, actstr);
		am.put(actstr,actBtnClicker);
	}
	
	class ButtonActionListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			action = e.getActionCommand();
			frame.setVisible(false);
            latchClose.countDown();
            frame.dispose();
		}
	}
	
	@SuppressWarnings("serial")
	class ButtonClickerAction extends AbstractAction {
		@Override
		public void actionPerformed(ActionEvent e) {
			JButton src = (JButton) e.getSource();
			src.doClick();
		}
	}

}
